#!/usr/bin/env ruby
# encoding: utf-8

#--
#   Copyright 2014 Luke Spangler
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#++

require "abc"

# This library contains a module for access
# to simple substitution cipher "encryption"
#
#
# Author::				Luke Spangler
# Email::				m27frogy.roblox@gmail.com

# Module containing the classes and methods
# of the library.
module SubstitutionCipher
	ALPHA_RANGE = 97..122
	# Remove non-alphabetical letters
	# from plaintext (bar spaces and newlines)
	def self.purge_nonalpha(text)
		text = text.dup.downcase
		table = []
		text.split("").each_with_index do |source,index|
			if not SubstitutionCipher::ALPHA_RANGE.include? source.ord and (not source.ord == 10 and not source.ord == 32) then
				table << index
			end
		end
		table.reverse.each do |index|
			text[index] = ""
		end
		return text
	end
	
	# "Encrypt" a string via a
	# substitution cipher.
	def self.encrypt(string,offset=3)
		string = self.purge_nonalpha string
		final = ""
		string.split("").each_with_index do |source,index|
			if (not source.ord == 10 and not source.ord == 32) then
				result = source.ord + offset
				if not SubstitutionCipher::ALPHA_RANGE.include? source.ord then
					result -= 25
				end
			else
				result = source.ord
			end
			final << result.chr
		end
		return final
	end
	
	# "Decrypt" a string via a
	# substitution cipher.
	def self.decrypt(string,offset=3)
		string = self.purge_nonalpha string
		final = ""
		string.split("").each_with_index do |source,index|
			if (not source.ord == 10 and not source.ord == 32) then
				result = source.ord - offset
				if not SubstitutionCipher::ALPHA_RANGE.include? source.ord then
					result += 25
				end
			else
				result = source.ord
			end
			final << result.chr
		end
		return final
	end
	
	# A method to crack (with the users help)
	# a string that has been "encrypted"
	# with SubstitutionCipher#encrypt.
	def self.crack_offset(string)
		first = Time.now
		range = 1..25
		complete = false
		range.to_a.each do |value|
			puts "\n-------------\n\n"
			puts self.decrypt(string,value)
			puts "\n-------------\n\n"
			puts "Valid? |Y N|"
			result = gets.chomp.downcase
			if result == "y" then
				complete = value
				break
			end
		end
		puts "\n-------------\n\n"
		if not complete then
			puts "Crack failed."
		else
			puts "Offset: #{complete}\nTime taken: #{Time.now-first}"
		end
		return complete
	end
end
